package org.gidea.scada.monitor.rabbit.listener;

import com.rabbitmq.client.BuiltinExchangeType;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.gidea.scada.monitor.rabbit.handler.RabbitMessageHandler;
import org.gidea.scada.monitor.utils.MonitorUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.config.SimpleRabbitListenerContainerFactory;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.ConcurrentReferenceHashMap;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.locks.Lock;

@Log4j2
@Component
public class MonitorRabbitListener {

    @Value("${org.gidea.monitor.client.name}")
    private String monitorName;

    private Map<String, SimpleMessageListenerContainer> queues = new ConcurrentReferenceHashMap<>();

    private Map<String, Lock> queueLocks = new ConcurrentReferenceHashMap<>();

    @Resource
    private SimpleRabbitListenerContainerFactory containerFactory;

    @Resource
    private Channel channel;


    private void queueListener(String deviceId, SimpleMessageListenerContainer container, String queueName, RabbitMessageHandler messageHandler) {
        container.setQueueNames(queueName);
        container.setMessageListener(new ChannelAwareMessageListener() {
            @Override
            public void onMessage(Message message, Channel channel) {
                String messageStr = new String(message.getBody());
                messageHandler.handleMessage(deviceId, messageStr);
            }
        });
        container.start();
    }

    /**
     * 设备实时数据监听
     *
     * @param deviceId       设备标识
     * @param messageHandler 设备消息处理
     */
    public void addDeviceListener(String deviceId, RabbitMessageHandler messageHandler) {
        addDeviceListener(deviceId, messageHandler, true);
    }

    /**
     * 设备数据监听
     *
     * @param deviceId       设备标识
     * @param messageHandler 设备消息处理
     * @param isReal         是否是实时数据 （true：实时数据，false：聚合数据）
     */
    public void addDeviceListener(String deviceId, RabbitMessageHandler messageHandler, boolean isReal) {
        String queueName = MonitorUtils.getRequestQueueName(deviceId, monitorName);
        // 创建监听设备请求队列
        SimpleMessageListenerContainer container = queues.get(queueName);
        if (container != null) {
            return;
        }
        try {
            String exchangeName = isReal ? MonitorUtils.getDeviceExchangeName(deviceId) : MonitorUtils.getJhExchangeName(deviceId);
            channel.exchangeDeclare(exchangeName, BuiltinExchangeType.FANOUT, false, false, null);
            channel.queueDeclare(queueName, true, true, true, null);
            channel.queueBind(queueName, exchangeName, deviceId);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        container = containerFactory.createListenerContainer();
        queueListener(deviceId, container, queueName, messageHandler);
        queues.put(queueName, container);
    }

    public void removeDeviceListener(String queueResponseName) {
        // 删除监听设备响应队列
        try {
            SimpleMessageListenerContainer container = queues.get(queueResponseName);
            if (container == null) {
                return;
            }
            container.stop();
            queues.remove(queueResponseName);
        } finally {
            // 删除设备锁信息
            queueLocks.remove(queueResponseName);
        }
    }


}
